@@ -1,4 +1,6 @@
+import copy
 from CyberSource.logging.log_configuration import LogConfiguration
+from authenticationsdk.util.CertificateUtility import CertificateUtility
 from authenticationsdk.util.GlobalLabelParameters import *
 from wsgiref.handlers import format_date_time
 from datetime import datetime
@@ -21,8 +23,6 @@ class MerchantConfiguration:
         self.host_name = None
         self.url = None
         self.request_host = None
-        self.request_type_method = None
-        self.request_target = None
         self.authentication_type = None
         self.key_file_path = None
         self.run_environment = None
@@ -46,15 +46,19 @@ class MerchantConfiguration:
         self.proxy_address = None
         self.proxy_port = None
         self.key_file_name = None
-        self.request_json_path_data = None
         self.solution_id = None
         self.log_config = None
         self.__jwePEMFileDirectory = None
         self.useMLEGlobally = None
+        self.enableRequestMLEForOptionalApisGlobally = None
+        self.disableRequestMLEForMandatoryApisGlobally = None
         self.mapToControlMLEonAPI = None
-        self.mleKeyAlias = None 
+        self.mleKeyAlias = None
+        self.mleForRequestPublicCertPath = None
+        self.p12KeyFilePath = None
         self.logger = LogFactory.setup_logger(self.__class__.__name__)
 
+#region Getters and Setters
     def set_merchant_keyid(self, value):
         if not (value.get('merchant_keyid') is None):
             self.merchant_keyid = value['merchant_keyid']
@@ -88,7 +92,7 @@ class MerchantConfiguration:
             self.use_metakey = value['use_metakey']
         else:
             self.use_metakey = False
-    
+
     def set_portfolio_id(self, value):
         if not (value.get('portfolio_id') is None):
             self.portfolio_id = value['portfolio_id']
@@ -138,7 +142,7 @@ class MerchantConfiguration:
             self.enable_client_cert = value['enable_client_cert']
         else:
             self.enable_client_cert = False
-    
+
     def set_client_cert_dir(self, value):
         if not (value.get('client_cert_dir') is None):
             self.client_cert_dir = value['client_cert_dir']
@@ -183,16 +187,38 @@ class MerchantConfiguration:
 
     def get_jwePEMFileDirectory(self):
         return self.__jwePEMFileDirectory
-    
+
     def set_useMLEGlobally(self, value):
         if not (value.get('useMLEGlobally') is None):
             self.useMLEGlobally = value['useMLEGlobally']
-        else:
-            self.useMLEGlobally = False
 
     def get_useMLEGlobally(self):
         return self.useMLEGlobally
     
+    def set_enableRequestMLEForOptionalApisGlobally(self, value):
+        enable_mle = value.get('enableRequestMLEForOptionalApisGlobally')
+        use_mle = value.get('useMLEGlobally')
+
+        if enable_mle is not None and use_mle is not None and enable_mle != use_mle:
+            raise ValueError("useMLEGlobally and enableRequestMLEForOptionalApisGlobally must have the same value.")
+
+        self.enableRequestMLEForOptionalApisGlobally = (
+            enable_mle if enable_mle is not None else (use_mle if use_mle is not None else False)
+        )
+
+    def get_enableRequestMLEForOptionalApisGlobally(self):
+        return self.enableRequestMLEForOptionalApisGlobally
+    
+
+    def set_disableRequestMLEForMandatoryApisGlobally(self, value):
+        self.disableRequestMLEForMandatoryApisGlobally = (
+            value.get('disableRequestMLEForMandatoryApisGlobally')
+            or False
+        )
+
+    def get_disableRequestMLEForMandatoryApisGlobally(self):
+        return self.disableRequestMLEForMandatoryApisGlobally
+    
     def set_mapToControlMLEonAPI(self, value):
         map_to_control_mle_on_api = value.get('mapToControlMLEonAPI')
         if map_to_control_mle_on_api is not None:
@@ -203,7 +229,7 @@ class MerchantConfiguration:
 
     def get_mapToControlMLEonAPI(self):
         return self.mapToControlMLEonAPI
-    
+
     def set_mleKeyAlias(self, value):
         if value.get('mleKeyAlias') is not None and value.get('mleKeyAlias').strip():
             self.mleKeyAlias = value['mleKeyAlias'].strip()
@@ -213,6 +239,25 @@ class MerchantConfiguration:
     def get_mleKeyAlias(self):
         return self.mleKeyAlias
 
+    def set_mleForRequestPublicCertPath(self, value):
+        if value.get('mleForRequestPublicCertPath') is not None and value.get('mleForRequestPublicCertPath').strip():
+            self.mleForRequestPublicCertPath = value['mleForRequestPublicCertPath'].strip()
+        else:
+            self.mleForRequestPublicCertPath = None
+
+    def get_mleForRequestPublicCertPath(self):
+        return self.mleForRequestPublicCertPath
+
+    def set_p12KeyFilePath(self, value):
+        if value.get('p12KeyFilePath') is not None and value.get('p12KeyFilePath').strip():
+            self.p12KeyFilePath = value['p12KeyFilePath'].strip()
+        else:
+            self.p12KeyFilePath = None
+
+    def get_p12KeyFilePath(self):
+        return self.p12KeyFilePath
+
+#endregion
 
     # This method sets the Merchant Configuration Variables to its respective values after reading from cybs.properties
     def set_merchantconfig(self, val):
@@ -246,7 +291,10 @@ class MerchantConfiguration:
         self.set_log_configuration(val)
         self.set_jwePEMFileDirectory(val)
         self.set_useMLEGlobally(val)
+        self.set_enableRequestMLEForOptionalApisGlobally(val)
+        self.set_disableRequestMLEForMandatoryApisGlobally(val)
         self.set_mapToControlMLEonAPI(val)
+        self.set_mleForRequestPublicCertPath(val)
         self.set_mleKeyAlias(val)
 
     # Returns the time in format as defined by RFC7231
@@ -281,12 +329,12 @@ class MerchantConfiguration:
                     authenticationsdk.util.ExceptionAuth.validate_merchant_details_log(self.logger,
                                                                                GlobalLabelParameters.CLIENT_CERT_DIR_EMPTY,
                                                                                self.log_config)
-            
+
             if self.ssl_client_cert is None or self.ssl_client_cert == "":
                     authenticationsdk.util.ExceptionAuth.validate_merchant_details_log(self.logger,
                                                                                GlobalLabelParameters.SSL_CLIENT_CERT_EMPTY,
                                                                                self.log_config)
-            
+
             if self.private_key is None or self.private_key == "":
                     authenticationsdk.util.ExceptionAuth.validate_merchant_details_log(self.logger,
                                                                                GlobalLabelParameters.PRIVATE_KEY_EMPTY,
@@ -302,7 +350,7 @@ class MerchantConfiguration:
                     authenticationsdk.util.ExceptionAuth.validate_merchant_details_log(self.logger,
                                                                                GlobalLabelParameters.MERCHANTID_REQ,
                                                                                self.log_config)
-                
+
                 if self.merchant_keyid is None or self.merchant_keyid == "":
                     authenticationsdk.util.ExceptionAuth.validate_merchant_details_log(self.logger,
                                                                                        GlobalLabelParameters.MERCHANT_KEY_ID_REQ,
@@ -318,7 +366,7 @@ class MerchantConfiguration:
                     authenticationsdk.util.ExceptionAuth.validate_merchant_details_log(self.logger,
                                                                                GlobalLabelParameters.MERCHANTID_REQ,
                                                                                self.log_config)
-                
+
                 if self.key_alias is None or self.key_alias == "":
                     self.key_alias = self.merchant_id
                     authenticationsdk.util.ExceptionAuth.validate_default_values(self.logger,
@@ -349,23 +397,26 @@ class MerchantConfiguration:
                                                                                  GlobalLabelParameters.KEY_FILE_EMPTY,
                                                                                  self.log_config)
 
-            elif self.authentication_type.lower() == GlobalLabelParameters.OAUTH.lower():                
+                if not self.check_key_file():
+                    authenticationsdk.util.ExceptionAuth.log_exception(self.logger, f"Error finding or accessing the Key Directory or Key File. Please review the values in the merchant configuration.", self.log_config)
+
+            elif self.authentication_type.lower() == GlobalLabelParameters.OAUTH.lower():
                 if self.access_token is None or self.access_token == "":
                     authenticationsdk.util.ExceptionAuth.validate_merchant_details_log(self.logger,
                                                                                GlobalLabelParameters.ACCESS_TOKEN_EMPTY,
                                                                                self.log_config)
-            
+
                 if self.ssl_client_cert is None or self.ssl_client_cert == "":
                     authenticationsdk.util.ExceptionAuth.validate_merchant_details_log(self.logger,
                                                                                GlobalLabelParameters.REFRESH_TOKEN_EMPTY,
                                                                                self.log_config)
-            
-            elif self.authentication_type.lower() == GlobalLabelParameters.MUTUAL_AUTH.lower():                
+
+            elif self.authentication_type.lower() == GlobalLabelParameters.MUTUAL_AUTH.lower():
                 if self.client_id is None or self.client_id == "":
                     authenticationsdk.util.ExceptionAuth.validate_merchant_details_log(self.logger,
                                                                                GlobalLabelParameters.CLIENT_ID_EMPTY,
                                                                                self.log_config)
-            
+
                 if self.client_secret is None or self.client_secret == "":
                     authenticationsdk.util.ExceptionAuth.validate_merchant_details_log(self.logger,
                                                                                GlobalLabelParameters.CLIENT_SECRET_EMPTY,
@@ -374,9 +425,16 @@ class MerchantConfiguration:
             authenticationsdk.util.ExceptionAuth.validate_merchant_details_log(self.logger,
                                                                                GlobalLabelParameters.AUTH_ERROR,
                                                                                self.log_config)
+            
+        use_mle = self.useMLEGlobally
+        enable_mle = self.enableRequestMLEForOptionalApisGlobally
+
+        if use_mle is not None and enable_mle is not None and use_mle != enable_mle:
+            raise ValueError("useMLEGlobally and enableRequestMLEForOptionalApisGlobally must have the same value.")
+        
         # useMLEGlobally check for auth Type
-        if self.useMLEGlobally is True or self.mapToControlMLEonAPI is not None:
-            if self.useMLEGlobally is True and self.authentication_type.lower() != GlobalLabelParameters.JWT.lower():
+        if self.enableRequestMLEForOptionalApisGlobally is True or self.mapToControlMLEonAPI is not None:
+            if self.enableRequestMLEForOptionalApisGlobally is True and self.authentication_type.lower() != GlobalLabelParameters.JWT.lower():
                  authenticationsdk.util.ExceptionAuth.validate_merchant_details_log(self.logger,
                                                                                GlobalLabelParameters.MLE_AUTH_ERROR,
                                                                                self.log_config)
@@ -388,14 +446,58 @@ class MerchantConfiguration:
                                                                                GlobalLabelParameters.MLE_AUTH_ERROR,
                                                                                self.log_config)
 
+        self.validate_MLE_configuration()
+        self.p12KeyFilePath = os.path.join(self.key_file_path, self.key_file_name) + GlobalLabelParameters.P12_PREFIX
+
         log_items = GlobalLabelParameters.HIDE_MERCHANT_CONFIG_PROPS
         # This displays the logic for logging all cybs.json values
+        details_copy = copy.deepcopy(details)
         if self.log_config.enable_log is True:
-            for key, value in list(details.items()):
+            for key, value in list(details_copy.items()):
                 if key in log_items:
-                    del details[key]
+                    del details_copy[key]
 
-                for keys, values in list(details.items()):
-                    details[keys] = str(values)
+                for keys, values in list(details_copy.items()):
+                    details_copy[keys] = str(values)
 
-            self.logger.info("Mconfig >      " + str(ast.literal_eval(json.dumps(details))))
+            self.logger.info("Mconfig >      " + str(ast.literal_eval(json.dumps(details_copy))))
+
+    def check_key_file(self):
+        if not(self.key_file_name and self.key_file_name.strip()):
+            self.logger.error("Key Filename not provided. Assigning the value of Merchant ID")
+            if self.merchant_id and self.merchant_id.strip():
+                self.key_file_name = self.merchant_id
+
+        if not(self.key_file_path and self.key_file_path.strip()):
+            self.key_file_path = GlobalLabelParameters.DEFAULT_KEY_FILE_PATH
+            self.logger.error(f"Keys Directory not provided. Using Default Path: {self.key_file_path}")
+
+        # Directory exists?
+        if not os.path.isdir(self.key_file_path):
+            self.logger.error(f"Keys Directory not found. Entered directory : {self.key_file_path}")
+            return False
+
+        keyFilePath = os.path.join(self.key_file_path, self.key_file_name) + GlobalLabelParameters.P12_PREFIX
+
+        # File exists?
+        if not os.path.isfile(keyFilePath):
+            self.logger.error(f"Key File not found. Check path/filename entered. Entered path/filename : {keyFilePath}")
+            return False
+
+        self.logger.info(f"Entered value for Key File Path : {keyFilePath}")
+
+        # Can file be opened for reading?
+        try:
+            with open(keyFilePath, 'rb'):
+                return True
+        except Exception:
+            self.logger.info(f"File cannot be accessed. Permission denied : {keyFilePath}")
+            return False
+
+    def validate_MLE_configuration(self):
+        if self.mleForRequestPublicCertPath and self.mleForRequestPublicCertPath.strip():
+            try:
+                CertificateUtility.validate_path_and_file(self.mleForRequestPublicCertPath, "mleForRequestPublicCertPath", self.log_config)
+            except Exception as err:
+                self.logger.error(str(err))
+                raise err
